﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace shuzihuarongdao
{
    /// <summary>
    /// MainWindow.xaml 的交互逻辑
    /// </summary>
    public partial class MainWindow : Window
    {
        Board board = new Board();
        enum GameState
        {
            FREE_STYLE,
            GENERATED,
            STARTED,
            FINISHED,
        }
        GameState gameState = GameState.FREE_STYLE;
        Stopwatch timer = new Stopwatch();
        DispatcherTimer uiTimer = new DispatcherTimer();
        int stepCount = 0;

        public MainWindow()
        {
            InitializeComponent();
            uiTimer.Tick += new EventHandler(UpdateTimerUI);
            uiTimer.Interval = new TimeSpan(0, 0, 0, 0, 100);  // 0.1s
        }

        private void UpdateTimerUI(object sender, EventArgs e)
        {
            TimerLabel.Content = timer.Elapsed.ToString(@"mm\:ss\.ff");
        }

        private void GenerateButton_Click(object sender, RoutedEventArgs e)
        {
            if (gameState == GameState.FREE_STYLE || gameState == GameState.FINISHED)
            {
                DifficultyLevel difficultyLevel = DifficultyLevel.NORMAL;
                if ((bool)Easy.IsChecked)
                {
                    difficultyLevel = DifficultyLevel.EASY;
                }
                if ((bool)Normal.IsChecked)
                {
                    difficultyLevel = DifficultyLevel.NORMAL;
                }
                if ((bool)Hard.IsChecked)
                {
                    difficultyLevel = DifficultyLevel.HARD;
                }
                board = Board.GenerateBoard(difficultyLevel);
                GenerateButton.Content = "开始";
                ResetGameStates();
                gameState = GameState.GENERATED;
                BlocksGrid.Visibility = Visibility.Hidden;
                DifficultySelect.IsEnabled = false;
            }
            else if (gameState == GameState.GENERATED)
            {
                GenerateButton.Content = "放弃";
                ShowBoard(board);
                timer.Start();
                uiTimer.Start();
                gameState = GameState.STARTED;
            }
            else if (gameState == GameState.STARTED)
            {
                GameFinished();
            }
        }

        private void ResetGameStates()
        {
            timer.Reset();
            stepCount = 0;
            TimerLabel.Content = "00:00.00";
            StepCountLabel.Content = stepCount;
        }

        /**
         * show board on GUI window
         */
        private void ShowBoard(Board board)
        {
            for (int i = 0; i < 16; i++)
            {
                string buttonName = getButtonName(i);
                Button button = (Button)MainGrid.FindName(buttonName);
                int label = board.GetPositionLabel(i);
                button.Content = label;
                if (label == 0)
                {
                    button.Visibility = Visibility.Hidden;
                }
                else
                {
                    button.Visibility = Visibility.Visible;
                }
            }
            BlocksGrid.Visibility = Visibility.Visible;
        }

        private void MoveBlockMaybe(object sender, RoutedEventArgs e)
        {
            if (gameState != GameState.STARTED)
            {
                return;
            }
            Button button = (Button)sender;
            int position = getBlockPosition(button.Name.ToString());
            // Console.WriteLine("position=" + position);
            string clickBlockLabel = button.Content.ToString();
            // Console.WriteLine("clickBlockLabel=" + clickBlockLabel);
            if (board.BlockCanMove(position))
            {
                string oldEmptyBlockName = getButtonName(board.GetEmptyBlockPosition());
                board.MoveBlock(position);
                string newEmptyBlockName = getButtonName(board.GetEmptyBlockPosition());
                Button newEmptyBlock = (Button)MainGrid.FindName(newEmptyBlockName);
                newEmptyBlock.Visibility = Visibility.Hidden;
                Button oldEmptyBlock = (Button)MainGrid.FindName(oldEmptyBlockName);
                oldEmptyBlock.Content = clickBlockLabel;
                oldEmptyBlock.Visibility = Visibility.Visible;

                IncrementStepCount();
                if (board.IsSolved())
                {
                    GameFinished();
                }
            }
        }

        private void GameFinished()
        {
            timer.Stop();
            uiTimer.Stop();
            // show final timer
            TimerLabel.Content = timer.Elapsed.ToString(@"mm\:ss\.ff");
            gameState = GameState.FINISHED;
            GenerateButton.Content = "新游戏";
            DifficultySelect.IsEnabled = true;
        }

        private void IncrementStepCount()
        {
            stepCount++;
            StepCountLabel.Content = stepCount;
        }

        /*
         * convert block position to button name. see also getBlockPosition().
         */
        private string getButtonName(int position)
        {
            return "Button" + position.ToString();
        }

        /*
         * this function map button name to block position.
         */
        private int getBlockPosition(String buttonName)
        {
            return Convert.ToInt32(buttonName.Substring(6));
        }
    }
}
